@@ -16,11 +16,12 @@
 
 """ Functions used to plot data on plot windows and to do other tasks. """
 
-from os import remove, path, mkdir
+from os import remove, path, mkdir, rmdir
 from time import sleep
 
 from .global_variables import *
 from .errors import *
+from .funcutils import correct_filename
 from .classes import _PlotWindow
 
 
@@ -47,6 +48,7 @@ def new_plot(*,
              xlabel=None, ylabel=None, zlabel=None,
              persistence=PERSISTENCE,
              redirect_output=REDIRECT_OUT,
+             purge=PURGE_DATA,
              options=None):        
     """ This function returns a plot as an istance of the class _PlotWindow.
 
@@ -55,7 +57,7 @@ def new_plot(*,
         Parameters
         ----------
 
-        (Note that all the parameters are keyword-only)
+        (All the parameters are keyword-only)
 
         xpos:            x position of the plot on the screen
         ypos:            y position of the plot on the screen
@@ -92,6 +94,8 @@ def new_plot(*,
         redirect_output: True:  save gnuplot otuput and errors to files
                          False: send them to /dev/stdout and /dev/stderr
                          None:  send them to /dev/null
+        purge:           if True, old datafiles are deleted each time 
+                         new data is plotted on this window
         options:         a string containing other options for the terminal
 
         .. [1] read gnuplot documentation for format strings
@@ -115,21 +119,23 @@ def new_plot(*,
     (status, message) = NOERROR
 
     if (gnuplot_installed is False):
-        raise FileNotFoundError('gnuplot is not installed on this system')
+        raise FileNotFoundError('gnuplot program was not found')
         
     # Create an instance of the _PlotWindow class
-    plot_window = _PlotWindow(plot_type=plot_type,
-                              gnuplot_default=gnuplot_default,
-                              terminal=terminal,
+    plot_window = _PlotWindow(gnuplot_default=gnuplot_default,
+                              terminal=terminal,                              
                               width=width,
                               height=height,
                               xpos=xpos,
                               ypos=ypos,
-                              title=title,
+                              options=options,                              
+                              plot_type=plot_type,
+                              title=title,                              
                               persistence=persistence,
-                              redirect_output=redirect_output)
-    
-    # If there were errors during the instance creation
+                              redirect_output=redirect_output,
+                              purge=purge)
+       
+    # If there were not errors during the instance creation,
     # do not apply the requested settings
     if not plot_window.error[0]:   
         # Apply the requested plot settings
@@ -218,7 +224,13 @@ def plot_set(plot,
 
     if not isinstance(plot, _PlotWindow): return ERROR_NOT_A_PLOT
     if (plot.plot_type is None): return ERROR_CLOSED_PLOT
-    if (replot and (not plot.data_filenames) and (not plot.functions)):
+    if (plot.n_volatiles
+        and ((logx is not None) or (logy is not None) or (logz is not None))):
+        return ERROR_LOGSET_VOLATILE    
+    if (replot
+        and (not plot.n_volatiles)
+        and (not plot.data_filenames) and
+        (not plot.functions)):
         return ERROR_NO_REPLOT
     if (style is not None ):
         if ( (plot.plot_type == '2D') and (style not in ALLOWED_STYLES_2D) ):
@@ -343,7 +355,6 @@ def plot_command(plot_window, string):
         ----------
 
         plot_window: the plot window to which the command must be sent
-
         string:      the string containing the gnuplot command
 
         Returns
@@ -365,7 +376,39 @@ def plot_command(plot_window, string):
 # | Functions to plot data |
 # +------------------------+
 
-def plot2d(plot_window, x_data, y_data, label=None, replot=False):
+def plot1d(plot_window, data, label=None, volatile=VOLATILE,replot=False):
+    """ Plots data on a previously initialized plot 
+
+        Parameters
+        ----------
+
+        plot_window:  the plot on which data should be plotted, which must 
+                      have been previously created by the new_plot function
+        data:         y-values of data points (x-values are automatically 
+                      calculated by gnuplot)
+        label:        string used to describe that set of data in the plot
+        volatile:     if True, the data are not written to file, but sent 
+                      to gnuplot as volatile data using the special filename '-'
+        replot:       add new plot instead of overwriting an old one
+
+        Returns
+        -------
+
+        One of the tuples defined in the errors.py module
+    """
+
+    if not isinstance(plot_window, _PlotWindow): return ERROR_NOT_A_PLOT    
+    if (plot_window.plot_type is None): return ERROR_CLOSED_PLOT
+    if (plot_window.plot_type == '3D'):
+        (status, message) = ERROR_WRONG_TYPE
+        message += ': 1D data on a 3D plot window'
+        return status, message
+    
+    return plot_window._add_curves([ [None, data, label, None] ], volatile, replot)        
+
+
+
+def plot2d(plot_window, x_data, y_data, label=None, volatile=VOLATILE, replot=False):
     """ Plots 2D data on a previously initialized plot 
 
         Parameters
@@ -376,6 +419,8 @@ def plot2d(plot_window, x_data, y_data, label=None, replot=False):
         x_data:       numpy array containing the x-values of data points
         y_data:       numpy array containing the y-values of data points
         label:        string used to describe that set of data in the plot
+        volatile:     if True, the data are not written to file, but sent 
+                      to gnuplot as volatile data using the special filename '-'
         replot:       add new plot instead of overwriting an old one
 
         Returns
@@ -391,10 +436,10 @@ def plot2d(plot_window, x_data, y_data, label=None, replot=False):
         message += ': 2D data on a 3D plot window'
         return status, message
     
-    return plot_window._add_curves([ [x_data, y_data, label, None] ], replot)        
+    return plot_window._add_curves([ [x_data, y_data, label, None] ], volatile, replot)        
 
         
-def plot3d(plot_window, x_data, y_data, z_data, label=None, replot=False):
+def plot3d(plot_window, x_data, y_data, z_data, label=None, volatile=VOLATILE, replot=False):
     """ Plots 3D data on a previously initialized plot 
 
         Parameters
@@ -406,6 +451,8 @@ def plot3d(plot_window, x_data, y_data, z_data, label=None, replot=False):
         y_data:       numpy array containing the y-values of data points
         z_data:       numpy array containing the z-values of data points
         label:        string used to describe that set of data in the plot
+        volatile:     if True, the data are not written to file, but sent 
+                      to gnuplot as volatile data using the special filename '-'
         replot:       add new plot instead of overwerwriting an old one
 
         Returns
@@ -421,10 +468,43 @@ def plot3d(plot_window, x_data, y_data, z_data, label=None, replot=False):
         message += ': 3D data on a 2D plot window'
         return status, message
     
-    return plot_window._add_curves([ [x_data, y_data, z_data, label, None] ], replot)
+    return plot_window._add_curves([ [x_data, y_data, z_data, label, None] ], volatile, replot)
+
+
+def plot_box(plot_window, data, width=None, label=None, volatile=VOLATILE, replot=False):
+    """ Plots a box plot from the given data on a previously initialized plot 
+
+        Parameters
+        ----------
+
+        plot_window:  the plot on which data should be plotted, which must 
+                      have been previously created by the new_plot function
+        data:         numpy array containing the data to be plotted
+        width:        width of the box, if not given is decided by gnuplot
+        label:        string used to describe that set of data in the plot
+        volatile:     if True, the data are not written to file, but sent 
+                      to gnuplot as volatile data using the special filename '-'
+        replot:       add new plot instead of overwerwriting an old one
+
+        Returns
+        -------
+
+        One of the tuples defined in the errors.py module
+    """
+    
+    if not isinstance(plot_window, _PlotWindow): return ERROR_NOT_A_PLOT
+    if (plot_window.plot_type is None): return ERROR_CLOSED_PLOT 
+    if (plot_window.plot_type != '2D'):
+        (status, message) = ERROR_WRONG_TYPE
+        message += ': 3D data on a 2D plot window'
+        return status, message
+    if (width is not None):
+        plot_window._command('set boxwidth ' + str(width))
+    
+    return plot_window._add_curves([ [None, data, label, 'with boxplot'] ], volatile, replot)
 
         
-def plot_curves(plot_window, data_list, replot=False):
+def plot_curves(plot_window, data_list, volatile=VOLATILE, replot=False):
     """ Plots several 2D or 3D data on a previously initialized plot 
 
         Parameters
@@ -438,16 +518,26 @@ def plot_curves(plot_window, data_list, replot=False):
                       3D data: [ [x1,  y1, z1, label1, options1], 
                                  [x2,  y2, z2, label2, options2], ... ] 
                       where:
-                      - x1 contains the x-coordinates of the points to plot
+                      - x1 contains the x-coordinates of the points to plot;
+                        for 2D plot windows this can be set to None, 
+                        which is useful if the data are 1D, as in the case 
+                        of boxplots, or if you want gnuplot to 
+                        automatically provide x-values.
+                        2D plots with and without x values can be mixed:
+                        [ [x1,   y1, label1, options1],     
+                          [None, y2, label2, options2],... ]  
                       - y1 contains the y-coordinates of the points to plot 
-                      - z1 contains the z-coordinates of the points to plot  
+                      - z1 contains the z-coordinates of the points to plot
+                        (for 3D plot windows only)  
                       - label1 is a string, that will be used to identify the plot 
                         in the legend, or None
-                      - options1 is a string, containing additional options
+                      - options1 is a string, containing additional options,
                         or None
                       The form must be consisted with the type of plot 
                       that was defined at the plot window creation, 
                       otherwise an error message is returned
+        volatile:     if True, the data are not written to file, but sent 
+                      to gnuplot as volatile data using the special filename '-'
         replot:       add new plots instead of overwerwriting an old ones
 
         Returns
@@ -459,7 +549,8 @@ def plot_curves(plot_window, data_list, replot=False):
     if not isinstance(plot_window, _PlotWindow): return ERROR_NOT_A_PLOT    
     if (plot_window.plot_type is None): return ERROR_CLOSED_PLOT
     
-    return plot_window._add_curves(data_list, replot)
+    return plot_window._add_curves(data_list, volatile, replot)
+
 
 
 # +------------------------------------------+
@@ -472,7 +563,7 @@ def plot_function(plot_window, func_string, label=None, replot=False):
         Parameters
         ----------
 
-        plot_window:  the plot on which data should be plotted, which must 
+        plot_window:  the plot, on which the function should be plotted, which must 
                       have been previously created by the new_plot function
         func_string:  a string containing the mathematical expression
                       using x (or x and y for 3D plots) as independent variable
@@ -481,7 +572,7 @@ def plot_function(plot_window, func_string, label=None, replot=False):
                       '2 * sin(x**2 + y**2)'
                       
                       The string is sent to gnuplot as it is, without checking 
-                      that it is a correct function al expression. If it is not, 
+                      that it is a correct functional expression. If it is not, 
                       gnuplot will provide  an error message, which will be
                       - sent to /dev/stderr, if the plot was opened with the 
                         'redirect_output=False' option (default)
@@ -511,8 +602,9 @@ def plot_functions(plot_window, func_list, replot=False):
 
         Parameters
         ----------
-        plot_window:  the plot on which data should be plotted, which must 
-                      have been previously created by the new_plot function
+
+        plot_window:  the plot, on which the functions should be plotted, which 
+                      must have been previously created by the new_plot function
         func_list:    a list in one the following form:
                       [ [function1, label1, options1], 
                         [function2, label2, options1], ... ]
@@ -525,7 +617,7 @@ def plot_functions(plot_window, func_list, replot=False):
                       - options1 is a string, containing additional options
                         or None                       
                       The string is sent to gnuplot as it is, without checking 
-                      that it is a correct function al expression. If it is not, 
+                      that it is a correct functional expression. If it is not, 
                       gnuplot will provide  an error message, which will be
                       - sent to /dev/stderr, if the plot was opened with the 
                         'redirect_output=False' option (default)
@@ -552,19 +644,25 @@ def plot_functions(plot_window, func_list, replot=False):
 
 def plot_print(plot_window,
                terminal=DEFAULT_PRINT_TERM,
+               dirname=None,
                filename=None,
                options=None):
     """ Export the plot to a file.
 
+        The plot is saved as an image, the type of which depends on the choosen 
+        terminal (default is png), in the present working directory.
+
         Parameters
         ----------
 
-        plot_window:    the plot on which data should be plotted, which must 
-                        have been previously created by the new_plot function
+        plot_window:    the plot window which must be printed
         terminal:       type of terminal to use to print the image 
-                        e.g. 'png', 'jpeg', 'gif'
+                        (e.g. 'png', 'jpeg', 'gif') mus be one of the
+                        terminal listed in the PRINT_TERMINALS global variable
+        dirname:        name of the directory where file must be saved;
+                        if not given, is the current working directory
         filename:       the file to which the image must be saved
-                        if non given, or set to None, a default one is used
+                        if not given, or set to None, a default one is used
         options:        a string with terminal options,
                         read gnuplot documentations for help
 
@@ -578,44 +676,61 @@ def plot_print(plot_window,
     
     if not isinstance(plot_window, _PlotWindow): return ERROR_NOT_A_PLOT
     if (plot_window.plot_type is None): return ERROR_CLOSED_PLOT
-    if ( (not plot_window.data_filenames) and (not plot_window.functions) ):
+    if ( (not plot_window.n_volatiles)
+         and (not plot_window.data_filenames)
+         and (not plot_window.functions) ):
         return ERROR_NO_REPLOT    
     
     if terminal not in PRINT_TERMINALS: return ERROR_UNKNOWN_TERM   
 
-    if not path.exists(DIRNAME_IMAGES): mkdir(DIRNAME_IMAGES)
     if (filename is None):
         filename = ( PRINT_FILENAME
                      + str(plot_window.window_number)
                      + PRINT_EXT[terminal] )
     else:
-        filename = str(filename)
-    
+        filename = correct_filename(str(filename))
+        
+    if (dirname is not None):
+        dirname = correct_filename(str(dirname))
+        if not path.exists(dirname): mkdir(dirname)
+        filename = path.join(dirname, filename)
+        
+    # Save the current terminal settings
     plot_window._command('set terminal push')
+
+    # Set the terminal for printing 
     command_string =  'set terminal '
     command_string += terminal
     if (options is not None):
         command_string += ' ' + str(options)    
     plot_window._command(command_string)
+
+    # Set the filename 
     command_string = 'set output '
-    filename = path.join(DIRNAME_IMAGES,
-                         plot_window._correct_filename(filename))
     command_string += '\"' + filename  + '\"'
+
+    # Sent the command to change terminal
     plot_window._command(command_string)
+
+    # Print the plot
     plot_window._command('replot')
+
+    # Recover the previous terminal settings
     plot_window._command('set terminal pop')
 
     return (status, message)
 
 
-def plot_print_all(terminal=DEFAULT_PRINT_TERM, options=None):
-    """ Esport all the open plot windows to files.
+def plot_print_all(terminal=DEFAULT_PRINT_TERM, dirname=None, options=None):
+    """ Export all the open plot windows to files.
 
         Parameters
         ----------
 
         terminal:       type of terminal to use to print the plots 
                         e.g. 'png', 'jpeg', 'gif'
+        dirname:        name of the directory where files must be saved;
+                        if not given, is the current working directory
         options:        a string with terminal options,
                         read gnuplot documentations for help
 
@@ -626,9 +741,14 @@ def plot_print_all(terminal=DEFAULT_PRINT_TERM, options=None):
     """
 
     (status, message) = NOERROR
+
+    if not window_list: return ERROR_NO_PLOTS    
        
     for plot_window in window_list:
-        plot_print(plot_window, terminal=terminal, options=options)
+        plot_print(plot_window,
+                   terminal=terminal,
+                   dirname=dirname,
+                   options=options)
 
     return (status, message)    
 
@@ -638,7 +758,7 @@ def plot_print_all(terminal=DEFAULT_PRINT_TERM, options=None):
 # +---------------------------------------------+
 
 def _best_pos(xmin, xmax, d=1):
-    """ Return a suitable coordinate to plot a dot.
+    """ Return coordinate to plot a dot to force gnuplot plot axis.
 
         Search for a suitable coordinate on an axis
         to plot a dot on the graph to force gnuplot 
@@ -650,7 +770,7 @@ def _best_pos(xmin, xmax, d=1):
         xmin: minimum value of the range
         xamx: maximum value of the range
         d:    distance from the extreme of the range
-              at which plot is dotted if some conditions
+              at which plot is plotted if some conditions
               are true
 
         Returns
@@ -702,8 +822,15 @@ def plot_reset(plot_window, plot_axes=True):
     if (plot_window.plot_type is None): return ERROR_CLOSED_PLOT
 
     # Reset functions and curves
+    if plot_window.purge:
+        for filename in plot_window.data_filenames:
+            try:
+                remove(filename)
+            except FileNotFoundError:
+                pass    
     plot_window.data_filenames.clear()
     plot_window.functions.clear()
+    plot_window.n_volatiles = 0
 
     # Remove labels
     plot_window._command('unset label')
@@ -741,7 +868,7 @@ def plot_reset_all(plot_axes=True):
     """ Resets all the plot windows.
 
         This function removes all the functions and curves
-        and clears all plot windows.
+        and clears all the plot windows.
 
         Parameters
         ----------
@@ -757,6 +884,8 @@ def plot_reset_all(plot_axes=True):
 
     (status, message) = NOERROR
 
+    if not window_list: return ERROR_NO_PLOTS    
+
     for plot_window in window_list:
         plot_reset(plot_window, plot_axes)
         
@@ -769,7 +898,7 @@ def plot_clear(plot_window):
         Parameters
         ----------
 
-        plot_window: the window (instance of the _PlotWindow class) to be cleared
+        plot_window: the window to be cleared
 
         Returns
         -------
@@ -794,13 +923,14 @@ def plot_clear_all():
         -------
 
         One of the tuples defined in the errors.py module
-
     """
 
     (status, message) = NOERROR
 
+    if not window_list: return ERROR_NO_PLOTS    
+
     for plot_window in window_list:
-        plot_clear(plot_window)
+        plot_window._command('clear')
         
     return status, message
 
@@ -824,7 +954,9 @@ def plot_replot(plot_window):
     if not isinstance(plot_window, _PlotWindow): return ERROR_NOT_A_PLOT
     if (plot_window.plot_type is None): return ERROR_CLOSED_PLOT
     
-    if ( (not plot_window.data_filenames) and (not plot_window.functions) ):
+    if ( (not plot_window.n_volatiles)
+         and (not plot_window.data_filenames)
+         and (not plot_window.functions) ):
         return ERROR_NO_REPLOT
     else:
         plot_window._command('replot')
@@ -841,9 +973,16 @@ def plot_replot_all():
         One of the tuples defined in the errors.py module
     """
     
-    (status, message) = NOERROR   
+    (status, message) = NOERROR
+
+    if not window_list: return ERROR_NO_PLOTS    
     
-    for plot_window in window_list: plot_replot(plot_window)
+    for plot_window in window_list:
+        (status1, message1) = plot_replot(plot_window)
+        if status1: (status, message) = (status1, message1)
+
+    if status:
+        message += ' on some plot windows' 
     
     return status, message
 
@@ -893,7 +1032,7 @@ def plot_label(plot_window,*, x=1, y=1, z=1, label=None, erase=False, replot=Fal
                              + str(x) + ',' + str(y) + ',' + str(z))
                             #+ 'left norotate front')
     plot_window._command('show label')
-    if replot: return plot_replot(plot_window)
+    if replot: plot_window._command('replot')
     
     return status, message
 
@@ -956,6 +1095,8 @@ def plot_raise_all():
     """
 
     (status, message) = NOERROR
+
+    if not window_list: return ERROR_NO_PLOTS    
     
     for plot_window in window_list: plot_window._command('raise')
     
@@ -972,6 +1113,8 @@ def plot_lower_all():
     """
 
     (status, message) = NOERROR
+
+    if not window_list: return ERROR_NO_PLOTS    
     
     for plot_window in window_list: plot_window._command('lower')
     
@@ -995,10 +1138,15 @@ def plot_check(plot_window, expanded=False, printout=True, getstring=False):
                      - window number
                      - type of terminal
                      - persistence status
+                     - purge status
                      - type of window ('2D', 'histogram', '3D')
                      - window title (if given)
                      - number of functions plotted
                      - number of curves plotted
+                     - number of volatile curves plotted
+                     - x-axis range
+                     - y-axis range
+                     - z-axis range (for 3D windows only)
                      if set to True, the following information is added
                      - PID of the gnuplot process
                      - names of data files to which the output and errors
@@ -1031,13 +1179,16 @@ def plot_check(plot_window, expanded=False, printout=True, getstring=False):
     if (plot_window.plot_type is None): return ERROR_CLOSED_PLOT   
     
     string = ''
+    string += 'Window index:         ' + str(window_list.index(plot_window)) + '\n'
     string += 'Window number:        ' + str(plot_window.window_number) + '\n'
     string += 'Terminal type:        ' + '\"' + plot_window.term_type + '\"' + '\n'
     string += 'Persistence:          ' + '\"' + str(plot_window.persistence) + '\"' + '\n'
+    string += 'Purge:                ' + '\"' + str(plot_window.purge) + '\"' + '\n'
     string += 'Window type:          ' + '\"' + str(plot_window.plot_type) + '\"' + '\n'   
     string += 'Window title:         ' + '\"' + str(plot_window.title) + '\"' + '\n'
-    string += 'Number of functions:  ' + str(len(plot_window.data_filenames)) + '\n'
-    string += 'Number of curves:     ' + str(len(plot_window.functions)) + '\n'
+    string += 'Number of functions:  ' + str(len(plot_window.functions)) + '\n'
+    string += 'Number of curves:     ' + str(len(plot_window.data_filenames)) + '\n'
+    string += 'Number of volatiles:  ' + str(plot_window.n_volatiles) + '\n'
     string += 'X-axis range:         ' + '[' + str(plot_window.xmin) + ',' + str(plot_window.xmax) + ']' + '\n'
     string += 'Y-axis range:         ' + '[' + str(plot_window.ymin) + ',' + str(plot_window.ymax) + ']' + '\n'
     if (plot_window.plot_type == '3D'):
@@ -1098,11 +1249,8 @@ def plot_list(expanded=False, printout=True, getstring=False):
 
     (status, message) = NOERROR
 
-    if len(window_list) == 0:
-        stsatus = 1
-        message = 'no plots'
-        return status, message
-    
+    if not window_list: return ERROR_NO_PLOTS
+   
     string = ''
     for plot_window in window_list:
         string += plot_check(plot_window, expanded, False, True)
@@ -1120,26 +1268,30 @@ def plot_list(expanded=False, printout=True, getstring=False):
 # | Functions to close active plots |
 # +---------------------------------+
             
-def plot_close(plot_window, purge=PURGE_FILES, delay=None):
+def plot_close(plot_window, keep_output=False, delay=None):
     """ Closes a plot windows and exits from the related gnuplot process 
 
+        If the plot window was created calling the new_plot function with 
+        the *purge=True* argument (which is the default) the data files 
+        of the presently plotted curves are deleted.
+
         Parameters
         ----------
 
-        plot_window: the plot to be closed (instance of _PlotWindow class)
-
-        purge:       if set to True, the data files used to store 
-                     data to be plotted will be removed
-        delay:       an int or float number, defining the time to
-                     wait (in secons) before deleting data files;
-                     this can be useful if a plot is created with
-                     the persist=True option and then immediately closed,
-                     since the gnuplot process could not be able to plot the
-                     data before the datafiles are deleted
+        plot_window:  the plot to be closed (instance of _PlotWindow class)
+        keep_output:  do not remove the files to which gnuplot output and 
+                      errors have been redirected, even if the window
+                      was created with the *purge=True* argument                 
+        delay:        an int or float number, defining the time to
+                      wait (in seconds) before deleting data files;
+                      this can be useful if a plot is created with
+                      the *persist=True* option and then immediately closed,
+                      since the gnuplot process could not be able to plot the
+                      data before the datafiles are deleted
         Returns
         -------
 
-       One of the tuples defined in the errors.py module
+        One of the tuples defined in the errors.py module
     """
 
     (status, message) = NOERROR
@@ -1147,40 +1299,38 @@ def plot_close(plot_window, purge=PURGE_FILES, delay=None):
     if not isinstance(plot_window, _PlotWindow): return ERROR_NOT_A_PLOT 
     if (plot_window.plot_type is None): return ERROR_CLOSED_PLOT    
 
-    # If requested, delete all the files associated to this plot
-    if purge:
-        
-        # Remove gnuplot output file
-        if (plot_window.filename_out not in ('/dev/stdout', '/dev/null')):
-            try:
-                remove(plot_window.filename_out)
-            except FileNotFoundError:
-                status = 1
-                message = ('the gnuplot output file for window # '
-                           + str(plot_window.window_number)
-                           + ' was not found')
-                
-        # Remove gnuplot error file                
-        if (plot_window.filename_err not in ('/dev/stderr', '/dev/null')):
-            try:
-                remove(plot_window.filename_err)
-            except FileNotFoundError:
-                status = 2
-                message = ('the gnuplot error file for window # '
-                           + str(plot_window.window_number)
-                           + ' was not found')
-                
+    # If window has purge attribute, delete the files associated to this plot
+    if plot_window.purge:
         # Remove data files but, if requested, wait a while before doing it 
         if (delay is not None): sleep(delay)
-        for i in range(len(plot_window.data_filenames)):
+        for filename in plot_window.data_filenames:
             try:
-                remove(plot_window.data_filenames[i])
+                remove(filename)
             except FileNotFoundError:
-                status = 3
-                message = ('one or more datafiles for window # '
-                           + str(plot_window.window_number)
-                           + ' were not found')
-                
+                (status, message) = ERROR_FILE
+                message += ('one or more datafiles for window # '
+                            + str(plot_window.window_number)
+                            + ' were not found')
+        if not keep_output:        
+            # Remove gnuplot output file
+            if (plot_window.filename_out not in ('/dev/stdout', '/dev/null')):
+                try:
+                    remove(plot_window.filename_out)
+                except FileNotFoundError:
+                    (status, message) = ERROR_FILE
+                    message += ('the gnuplot output file for window # '
+                                + str(plot_window.window_number)
+                                + ' was not found')
+            # Remove gnuplot error file                
+            if (plot_window.filename_err not in ('/dev/stderr', '/dev/null')):
+                try:
+                    remove(plot_window.filename_err)
+                except FileNotFoundError:
+                    (status, message) = ERROR_FILE
+                    message += ('the gnuplot error file for window # '
+                                + str(plot_window.window_number)
+                                + ' was not found')
+                    
     # Shut down the associated gnuplot process
     plot_window._quit_gnuplot()
 
@@ -1193,20 +1343,28 @@ def plot_close(plot_window, purge=PURGE_FILES, delay=None):
     return status, message
 
 
-def plot_close_all(purge=PURGE_FILES, delay=None):
-    """ Closes all plots 
+def plot_close_all(keep_output=False, delay=None, purge_dir=PURGE_DIR):
+    """ Closes all plots.
+
+        For plot windows which were created calling the new_plot function with 
+        the *purge=True* argument (which is the default), the data files 
+        of the presently plotted curves are deleted.
 
         Parameters
         ----------
 
-        purge:       if set to True, the data files used to store 
-                     data to be plotted will be removed 
-        delay:       an int or float number, defining the time to
-                     wait (in seconds) before deleting data files;
-                     this can be useful if a plot is created with
-                     the persist=True option and then immediately closed,
-                     since the gnuplot process could not be able to plot the
-                     data before the datafiles are deleted
+        keep_output:  do not remove the files to which gnuplot output and 
+                      errors have been redirected, even if the window
+                      was created with the *purge=True* argument
+        delay:        an int or float number, defining the time to
+                      wait (in seconds) before deleting data files;
+                      this can be useful if a plot is created with
+                      the persist=True option and then immediately closed,
+                      since the gnuplot process could not be able to plot the
+                      data before the datafiles are deleted
+        purge_dir:    remove the directories in which files are stored; 
+                      this works only if they are empty and is uneffective
+                      if keep_output has bee set to True
         Returns
         -------
 
@@ -1215,12 +1373,43 @@ def plot_close_all(purge=PURGE_FILES, delay=None):
 
     (status, message) = NOERROR
     
-    if len(window_list) == 0:
-        status = 1
-        message = 'there are no open plots'
-        return status, message
-    while (len(window_list) > 0):
+    while window_list:
         plot_window = window_list[-1]
-        (status, message) = plot_close(plot_window, purge, delay)
+        (status1, message1) = plot_close(plot_window, keep_output, delay)
+        # If there was an error closing the window, store it
+        if status1: (status, message) = (status1, message1)
+
+    if (purge_dir and not keep_output):
+        # Try to remove data file subdirectory
+        try:
+            rmdir(DIRNAME_DATA)
+        except FileNotFoundError:
+            pass
+        except OSError:
+            (status, message) = ERROR_FILE
+        # Try to remove subdirectory with gnuplot output            
+        try:
+            rmdir(DIRNAME_OUT)
+        except FileNotFoundError:
+            pass
+        except OSError:
+            (status, message) = ERROR_FILE
+        # Try to remove subdirectory with gnuplot errors            
+        try:
+            rmdir(DIRNAME_ERR)
+        except FileNotFoundError:
+            pass
+        except OSError:
+            (status, message) = ERROR_FILE
+        # Try to remove main directory
+        try:
+            rmdir(DIRNAME)
+        except FileNotFoundError:
+            pass
+        except OSError:
+            (status, message) = ERROR_FILE
                 
+    if status:
+        message += ': some files or directories could not be deleted'                
+  
     return status, message
